'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.FormProvider = exports.FormEventDispatcher = exports.REACT_FORMCTRL = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var REACT_FORMCTRL_NAME = 'react-formctl';

var INTEGER_REGEX = /^-?\d+?$/;
var FLOAT_REGEX = /^-?\d+(\.\d+)?$/;
var EMAIL_REGEX = /\S+@\S+\.\S+/;

var PROVIDER_FLAG = REACT_FORMCTRL_NAME + '.FormProvider';

var REACT_FORMCTRL = exports.REACT_FORMCTRL = {
    EVENTS: {
        REGISTER_FORM: REACT_FORMCTRL_NAME + '.registerForm',
        UNREGISTER_FORM: REACT_FORMCTRL_NAME + '.unregisterForm',
        REGISTER_FIELD: REACT_FORMCTRL_NAME + '.registerField',
        UNREGISTER_FIELD: REACT_FORMCTRL_NAME + '.unregisterField',
        FORM_CHANGED: REACT_FORMCTRL_NAME + '.formChanged',
        FIELD_CHANGED: REACT_FORMCTRL_NAME + '.fieldChanged',
        FIELD_PROPS_CHANGED: REACT_FORMCTRL_NAME + '.fieldPropsChanged',
        FIELD_BLURRED: REACT_FORMCTRL_NAME + '.fieldBlurred',
        FORM_SUBMITED: REACT_FORMCTRL_NAME + '.formSubmited',
        FORM_RESETED: REACT_FORMCTRL_NAME + '.formReseted'
    }
};

var FormEventDispatcher = exports.FormEventDispatcher = function () {
    function FormEventDispatcher() {
        _classCallCheck(this, FormEventDispatcher);
    }

    _createClass(FormEventDispatcher, null, [{
        key: 'copyFiles',
        value: function copyFiles(selectedFiles) {
            if (selectedFiles) {
                if (Array.isArray(selectedFiles)) {
                    return FormEventDispatcher.copyArray(selectedFiles);
                } else {
                    var files = [];
                    for (var i = 0; i < selectedFiles.length; i++) {
                        var file = selectedFiles.item(i);
                        files.push(file);
                    }
                    return files;
                }
            }
            return [];
        }
    }, {
        key: 'copyArray',
        value: function copyArray(array) {
            if (array) {
                return array.map(function (item) {
                    return item;
                });
            }
            return [];
        }
    }, {
        key: 'copyFieldCtrlProps',
        value: function copyFieldCtrlProps(props) {
            return {
                name: props.name,
                form: props.form,
                type: props.type,
                required: props.required,
                pattern: props.pattern,
                match: props.match,
                integer: props.integer,
                min: props.min,
                max: props.max,
                minLength: props.minLength,
                maxLength: props.maxLength,
                accept: props.accept,
                extensions: FormEventDispatcher.copyArray(props.extensions),
                maxSize: props.maxSize,
                validate: props.validate
            };
        }
    }, {
        key: 'copyError',
        value: function copyError(error) {
            var output = {
                key: error.key
            };
            if (error.params) {
                var params = {};
                Object.keys(error.params).forEach(function (paramName) {
                    var param = error.params[paramName];
                    if (Array.isArray(param)) params[paramName] = FormEventDispatcher.copyArray(param);else if (param instanceof FileList) params[paramName] = FormEventDispatcher.copyFiles(param);else params[paramName] = param;
                });
                output.params = params;
            }
            return output;
        }
    }, {
        key: 'copyErrors',
        value: function copyErrors(errors) {
            if (errors && errors.length) {
                return errors.map(function (error) {
                    return FormEventDispatcher.copyError(error);
                });
            }
            return errors;
        }
    }, {
        key: 'copyFieldCtrl',
        value: function copyFieldCtrl(fieldCtrl) {
            return {
                valid: fieldCtrl.valid,
                invalid: fieldCtrl.invalid,
                untouched: fieldCtrl.untouched,
                touched: fieldCtrl.touched,
                pristine: fieldCtrl.pristine,
                dirty: fieldCtrl.dirty,
                unchanged: fieldCtrl.unchanged,
                changed: fieldCtrl.changed,
                errors: FormEventDispatcher.copyErrors(fieldCtrl.errors),
                value: fieldCtrl.value,
                files: fieldCtrl.files,
                initialValue: fieldCtrl.initialValue,
                props: FormEventDispatcher.copyFieldCtrlProps(fieldCtrl.props)
            };
        }
    }, {
        key: 'copyFormValues',
        value: function copyFormValues(values) {
            var cValues = {};
            if (values) {
                var fieldsNames = Object.keys(values);
                if (fieldsNames.length) {
                    fieldsNames.forEach(function (fieldName) {
                        cValues[fieldName] = values[fieldName];
                    });
                }
            }
            return cValues;
        }
    }, {
        key: 'copyFormFiles',
        value: function copyFormFiles(files) {
            var cFiles = {};
            if (files) {
                var fieldsNames = Object.keys(files);
                if (fieldsNames.length) {
                    fieldsNames.forEach(function (fieldName) {
                        cFiles[fieldName] = FormEventDispatcher.copyFiles(files[fieldName]);
                    });
                }
            }
            return cFiles;
        }
    }, {
        key: 'copyFormFields',
        value: function copyFormFields(fields) {
            var cFields = {};
            if (fields) {
                var fieldsNames = Object.keys(fields);
                if (fieldsNames.length) {
                    fieldsNames.forEach(function (fieldName) {
                        cFields[fieldName] = FormEventDispatcher.copyFieldCtrl(fields[fieldName]);
                    });
                }
            }
            return cFields;
        }
    }, {
        key: 'copyFormCtrl',
        value: function copyFormCtrl(formCtrl) {
            return {
                valid: formCtrl.valid,
                invalid: formCtrl.invalid,
                untouched: formCtrl.untouched,
                touched: formCtrl.touched,
                pristine: formCtrl.pristine,
                dirty: formCtrl.dirty,
                unchanged: formCtrl.unchanged,
                changed: formCtrl.changed,
                fields: FormEventDispatcher.copyFormFields(formCtrl.fields),
                values: FormEventDispatcher.copyFormValues(formCtrl.values),
                files: FormEventDispatcher.copyFormFiles(formCtrl.files)
            };
        }
    }, {
        key: 'dispatchRegisterForm',
        value: function dispatchRegisterForm(form) {
            var payload = { detail: { form: form } };
            var event = new CustomEvent(REACT_FORMCTRL.EVENTS.REGISTER_FORM, payload);
            document.dispatchEvent(event);
        }
    }, {
        key: 'dispatchUnregisterForm',
        value: function dispatchUnregisterForm(form) {
            var payload = { detail: { form: form } };
            var event = new CustomEvent(REACT_FORMCTRL.EVENTS.UNREGISTER_FORM, payload);
            document.dispatchEvent(event);
        }
    }, {
        key: 'dispatchSubmitForm',
        value: function dispatchSubmitForm(form, formRef) {
            var payload = { detail: { form: form, formRef: formRef } };
            var event = new CustomEvent(REACT_FORMCTRL.EVENTS.FORM_SUBMITED, payload);
            document.dispatchEvent(event);
        }
    }, {
        key: 'dispatchResetForm',
        value: function dispatchResetForm(form) {
            var payload = { detail: { form: form } };
            var event = new CustomEvent(REACT_FORMCTRL.EVENTS.FORM_RESETED, payload);
            document.dispatchEvent(event);
        }
    }, {
        key: 'dispatchRegisterField',
        value: function dispatchRegisterField(form, field, fieldCtrl) {
            var payload = { detail: { form: form, field: field, fieldCtrl: FormEventDispatcher.copyFieldCtrl(fieldCtrl) } };
            var event = new CustomEvent(REACT_FORMCTRL.EVENTS.REGISTER_FIELD, payload);
            document.dispatchEvent(event);
        }
    }, {
        key: 'dispatchUnregisterField',
        value: function dispatchUnregisterField(form, field) {
            var payload = { detail: { form: form, field: field } };
            var event = new CustomEvent(REACT_FORMCTRL.EVENTS.UNREGISTER_FIELD, payload);
            document.dispatchEvent(event);
        }
    }, {
        key: 'dispatchFieldPropsChanged',
        value: function dispatchFieldPropsChanged(form, field, props) {
            var payload = { detail: { form: form, field: field, props: FormEventDispatcher.copyFieldCtrlProps(props) } };
            var event = new CustomEvent(REACT_FORMCTRL.EVENTS.FIELD_PROPS_CHANGED, payload);
            document.dispatchEvent(event);
        }
    }, {
        key: 'dispatchFieldChanged',
        value: function dispatchFieldChanged(form, field, value, files) {
            var payload = { detail: { form: form, field: field, value: value, files: FormEventDispatcher.copyFiles(files) } };
            var event = new CustomEvent(REACT_FORMCTRL.EVENTS.FIELD_CHANGED, payload);
            document.dispatchEvent(event);
        }
    }, {
        key: 'dispatchFieldBlur',
        value: function dispatchFieldBlur(form, field) {
            var payload = { detail: { form: form, field: field } };
            var event = new CustomEvent(REACT_FORMCTRL.EVENTS.FIELD_BLURRED, payload);
            document.dispatchEvent(event);
        }
    }, {
        key: 'forwardSubmitFormEvent',
        value: function forwardSubmitFormEvent(form, values, formCtrl, formRef) {
            var payload = { detail: { values: values, formRef: formRef, formCtrl: FormEventDispatcher.copyFormCtrl(formCtrl) } };
            var event = new CustomEvent(REACT_FORMCTRL.EVENTS.FORM_SUBMITED + '#' + form, payload);
            document.dispatchEvent(event);
        }
    }, {
        key: 'forwardFieldChangedEvent',
        value: function forwardFieldChangedEvent(form, field, fieldCtrl) {
            var payload = { detail: { form: form, field: field, fieldCtrl: FormEventDispatcher.copyFieldCtrl(fieldCtrl) } };
            var event = new CustomEvent(REACT_FORMCTRL.EVENTS.FIELD_CHANGED + '#' + form + '#' + field, payload);
            document.dispatchEvent(event);
        }
    }, {
        key: 'forwardFormChangedEvent',
        value: function forwardFormChangedEvent(form, formCtrl) {
            var payload = { detail: { form: form, formCtrl: FormEventDispatcher.copyFormCtrl(formCtrl) } };
            var event = new CustomEvent(REACT_FORMCTRL.EVENTS.FORM_CHANGED + '#' + form, payload);
            document.dispatchEvent(event);
        }
    }]);

    return FormEventDispatcher;
}();

var FormProvider = exports.FormProvider = function (_React$Component) {
    _inherits(FormProvider, _React$Component);

    function FormProvider(props) {
        _classCallCheck(this, FormProvider);

        var _this = _possibleConstructorReturn(this, (FormProvider.__proto__ || Object.getPrototypeOf(FormProvider)).call(this, props));

        _this.state = {
            forms: {},
            customValidators: {}
        };
        _this.subscribe = _this.subscribe.bind(_this);
        _this.unsubscribe = _this.unsubscribe.bind(_this);
        _this.onEvent = _this.onEvent.bind(_this);
        _this.onRegisterForm = _this.onRegisterForm.bind(_this);
        _this.onUnregisterForm = _this.onUnregisterForm.bind(_this);
        _this.onRegisterField = _this.onRegisterField.bind(_this);
        _this.onUnregisterField = _this.onUnregisterField.bind(_this);
        _this.onFieldChanged = _this.onFieldChanged.bind(_this);
        _this.onFieldPropsChanged = _this.onFieldPropsChanged.bind(_this);
        _this.onFieldBlurred = _this.onFieldBlurred.bind(_this);
        _this.onFormSubmited = _this.onFormSubmited.bind(_this);
        _this.onFormReseted = _this.onFormReseted.bind(_this);
        _this.updateFormCtrl = _this.updateFormCtrl.bind(_this);
        _this.updateFieldCtrl = _this.updateFieldCtrl.bind(_this);
        return _this;
    }

    _createClass(FormProvider, [{
        key: 'componentWillMount',
        value: function componentWillMount() {
            // if (window.sessionStorage.getItem(PROVIDER_FLAG)) {
            //     throw 'Two instances of FormProvided found, only one instance can exists because events erros may occur.'
            // }
            // window.sessionStorage.setItem(PROVIDER_FLAG, 'true')
            this.subscribe();
            var newState = {
                customValidators: {}
            };
            var _props$customValidato = this.props.customValidators,
                customValidators = _props$customValidato === undefined ? [] : _props$customValidato;

            customValidators.forEach(function (validator) {
                newState.customValidators[validator.name] = validator.validate;
            });
            this.setState(newState);
        }
    }, {
        key: 'componentWillUnmount',
        value: function componentWillUnmount() {
            // window.sessionStorage.removeItem(PROVIDER_FLAG)
            this.unsubscribe();
        }
    }, {
        key: 'subscribe',
        value: function subscribe() {
            document.addEventListener(REACT_FORMCTRL.EVENTS.REGISTER_FORM, this.onEvent);
            document.addEventListener(REACT_FORMCTRL.EVENTS.REGISTER_FIELD, this.onEvent);
            document.addEventListener(REACT_FORMCTRL.EVENTS.UNREGISTER_FORM, this.onEvent);
            document.addEventListener(REACT_FORMCTRL.EVENTS.UNREGISTER_FIELD, this.onEvent);
            document.addEventListener(REACT_FORMCTRL.EVENTS.FIELD_CHANGED, this.onEvent);
            document.addEventListener(REACT_FORMCTRL.EVENTS.FIELD_BLURRED, this.onEvent);
            document.addEventListener(REACT_FORMCTRL.EVENTS.FIELD_PROPS_CHANGED, this.onEvent);
            document.addEventListener(REACT_FORMCTRL.EVENTS.FORM_SUBMITED, this.onEvent);
            document.addEventListener(REACT_FORMCTRL.EVENTS.FORM_RESETED, this.onEvent);
        }
    }, {
        key: 'onEvent',
        value: function onEvent(event) {
            var type = event.type;
            var payload = event.detail;
            var EVENTS = REACT_FORMCTRL.EVENTS;
            switch (type) {
                case EVENTS.REGISTER_FORM:
                    this.onRegisterForm(payload.form);
                    break;
                case EVENTS.UNREGISTER_FORM:
                    this.onUnregisterForm(payload.form);
                    break;
                case EVENTS.REGISTER_FIELD:
                    this.onRegisterField(payload.form, payload.field, payload.fieldCtrl);
                    break;
                case EVENTS.UNREGISTER_FIELD:
                    this.onUnregisterField(payload.form, payload.field);
                    break;
                case EVENTS.FIELD_CHANGED:
                    this.onFieldChanged(payload.form, payload.field, payload.value, payload.files);
                    break;
                case EVENTS.FIELD_PROPS_CHANGED:
                    this.onFieldPropsChanged(payload.form, payload.field, payload.props);
                    break;
                case EVENTS.FIELD_BLURRED:
                    this.onFieldBlurred(payload.form, payload.field);
                    break;
                case EVENTS.FORM_SUBMITED:
                    this.onFormSubmited(payload.form, payload.formRef);
                    break;
                case EVENTS.FORM_RESETED:
                    this.onFormReseted(payload.form);
                    break;
                default:
                    break;
            }
        }
    }, {
        key: 'onRegisterForm',
        value: function onRegisterForm(formName) {
            this.setState(function (state) {
                var forms = _extends({}, state.forms);
                if (forms[formName]) {
                    var form = forms[formName];
                    form.__instances++;
                } else {
                    forms[formName] = {
                        __instances: 1,
                        formName: formName,
                        validating: false,
                        valid: true,
                        invalid: false,
                        untouched: true,
                        touched: false,
                        pristine: true,
                        dirty: false,
                        unchanged: true,
                        changed: false,
                        fields: {},
                        values: {},
                        files: {}
                    };
                }
                return { forms: forms };
            });
        }
    }, {
        key: 'onRegisterField',
        value: function onRegisterField(formName, fieldName, fieldCtrl) {
            var _this2 = this;

            this.setState(function (state) {
                if (state.forms[formName]) {
                    var forms = _extends({}, state.forms);
                    var form = forms[formName];
                    if (form.fields[fieldName]) {
                        fieldCtrl.__instances = form.fields[fieldName].__instances + 1;
                    } else {
                        fieldCtrl.__instances = 1;
                    }
                    _this2.updateFieldCtrl(formName, fieldCtrl, fieldCtrl.initialValue);
                    FormEventDispatcher.forwardFieldChangedEvent(formName, fieldName, fieldCtrl);

                    form.fields[fieldName] = fieldCtrl;
                    form.values[fieldName] = fieldCtrl.value;
                    _this2.updateFormCtrl(formName, form);
                    return { forms: forms };
                } else {
                    console.warn('No form instance with name "' + formName + '" to register field "' + fieldName + '".');
                }
                return state;
            });
        }
    }, {
        key: 'onUnregisterForm',
        value: function onUnregisterForm(formName) {
            this.setState(function (state) {
                if (state.forms[formName]) {
                    var forms = _extends({}, state.forms);
                    var form = forms[formName];
                    if (form.__instances > 1) {
                        form.__instances--;
                    } else {
                        delete forms[formName];
                    }
                    return { forms: forms };
                }
                return state;
            });
        }
    }, {
        key: 'onUnregisterField',
        value: function onUnregisterField(formName, fieldName) {
            this.setState(function (state) {
                if (state.forms[formName]) {
                    if (state.forms[formName].fields[fieldName]) {
                        var forms = _extends({}, state.forms);
                        var form = forms[formName];
                        var field = form.fields[fieldName];
                        if (field.__instances > 1) {
                            field.__instances--;
                        } else {
                            delete form.fields[fieldName];
                        }
                        return { forms: forms };
                    }
                }
                return state;
            });
        }
    }, {
        key: 'onFieldChanged',
        value: function onFieldChanged(formName, fieldName, value, files) {
            var _this3 = this;

            this.setState(function (state) {
                if (state.forms[formName]) {
                    if (state.forms[formName].fields[fieldName]) {
                        var forms = _extends({}, state.forms);
                        var form = forms[formName];
                        var fieldCtrl = form.fields[fieldName];
                        fieldCtrl.dirty = true;
                        fieldCtrl.pristine = false;
                        _this3.updateFieldCtrl(formName, fieldCtrl, value, files);
                        FormEventDispatcher.forwardFieldChangedEvent(formName, fieldName, fieldCtrl);
                        if (!form.values) form.values = {};
                        form.values[fieldName] = value;
                        if (!form.files) form.files = {};
                        if (files) {
                            form.files[fieldName] = files;
                        } else if (form.files[fieldName]) {
                            delete form.files[fieldName];
                        }
                        _this3.updateFormCtrl(formName, form);
                        return { forms: forms };
                    }
                }
                return state;
            });
        }
    }, {
        key: 'onFieldPropsChanged',
        value: function onFieldPropsChanged(formName, fieldName, props) {
            var _this4 = this;

            this.setState(function (state) {
                var forms = _extends({}, state.forms);
                var form = forms[formName];
                var field = form.fields[fieldName];
                field.props = props;
                _this4.updateFieldCtrl(formName, field, field.value);
                FormEventDispatcher.forwardFieldChangedEvent(formName, fieldName, field);
                _this4.updateFormCtrl(formName, form);
                return { forms: forms };
            });
        }
    }, {
        key: 'onFieldBlurred',
        value: function onFieldBlurred(formName, fieldName) {
            var _this5 = this;

            this.setState(function (state) {
                var forms = _extends({}, state.forms);
                var form = forms[formName];
                var field = form.fields[fieldName];
                field.touched = true;
                field.untouched = false;
                FormEventDispatcher.forwardFieldChangedEvent(formName, fieldName, field);
                _this5.updateFormCtrl(formName, form);
                return { forms: forms };
            });
        }
    }, {
        key: 'updateFormCtrl',
        value: function updateFormCtrl(formName, form) {
            form.validating = false;
            form.valid = true;
            form.invalid = false;
            form.untouched = true;
            form.touched = false;
            form.pristine = true;
            form.dirty = false;
            form.unchanged = true;
            form.changed = false;
            Object.keys(form.fields).forEach(function (fieldName) {
                var field = form.fields[fieldName];
                if (field.validating) form.validating = true;
                if (!field.valid) form.valid = false;
                if (field.invalid) form.invalid = true;
                if (!field.untouched) form.untouched = false;
                if (field.touched) form.touched = true;
                if (!field.pristine) form.pristine = false;
                if (field.dirty) form.dirty = true;
                if (!field.unchanged) form.unchanged = false;
                if (field.changed) form.changed = true;
            });
            FormEventDispatcher.forwardFormChangedEvent(formName, form);
        }
    }, {
        key: 'createValidationError',
        value: function createValidationError(key, params) {
            return {
                key: key,
                params: params
            };
        }
    }, {
        key: 'updateFieldCtrl',
        value: function updateFieldCtrl(formName, fieldCtrl, value, files) {
            var _this6 = this;

            var errors = [];
            var _state$customValidato = this.state.customValidators,
                customValidators = _state$customValidato === undefined ? {} : _state$customValidato;
            var initialValue = fieldCtrl.initialValue,
                _fieldCtrl$props = fieldCtrl.props,
                _fieldCtrl$props$name = _fieldCtrl$props.name,
                name = _fieldCtrl$props$name === undefined ? null : _fieldCtrl$props$name,
                _fieldCtrl$props$type = _fieldCtrl$props.type,
                type = _fieldCtrl$props$type === undefined ? 'text' : _fieldCtrl$props$type,
                _fieldCtrl$props$requ = _fieldCtrl$props.required,
                required = _fieldCtrl$props$requ === undefined ? null : _fieldCtrl$props$requ,
                _fieldCtrl$props$patt = _fieldCtrl$props.pattern,
                pattern = _fieldCtrl$props$patt === undefined ? null : _fieldCtrl$props$patt,
                _fieldCtrl$props$matc = _fieldCtrl$props.match,
                match = _fieldCtrl$props$matc === undefined ? null : _fieldCtrl$props$matc,
                _fieldCtrl$props$inte = _fieldCtrl$props.integer,
                integer = _fieldCtrl$props$inte === undefined ? null : _fieldCtrl$props$inte,
                _fieldCtrl$props$min = _fieldCtrl$props.min,
                min = _fieldCtrl$props$min === undefined ? null : _fieldCtrl$props$min,
                _fieldCtrl$props$max = _fieldCtrl$props.max,
                max = _fieldCtrl$props$max === undefined ? null : _fieldCtrl$props$max,
                _fieldCtrl$props$minL = _fieldCtrl$props.minLength,
                minLength = _fieldCtrl$props$minL === undefined ? null : _fieldCtrl$props$minL,
                _fieldCtrl$props$maxL = _fieldCtrl$props.maxLength,
                maxLength = _fieldCtrl$props$maxL === undefined ? null : _fieldCtrl$props$maxL,
                _fieldCtrl$props$exte = _fieldCtrl$props.extensions,
                extensions = _fieldCtrl$props$exte === undefined ? [] : _fieldCtrl$props$exte,
                _fieldCtrl$props$acce = _fieldCtrl$props.accept,
                accept = _fieldCtrl$props$acce === undefined ? null : _fieldCtrl$props$acce,
                _fieldCtrl$props$maxS = _fieldCtrl$props.maxSize,
                maxSize = _fieldCtrl$props$maxS === undefined ? null : _fieldCtrl$props$maxS,
                _fieldCtrl$props$vali = _fieldCtrl$props.validate,
                validate = _fieldCtrl$props$vali === undefined ? [] : _fieldCtrl$props$vali;


            if (required && !value) errors.push(this.createValidationError('required'));else if (pattern != null && pattern instanceof RegExp && !pattern.test(value)) errors.push(this.createValidationError('pattern', { value: value, pattern: pattern }));else if (pattern != null && !new RegExp(pattern).test(value)) errors.push(this.createValidationError('pattern', { value: value, pattern: pattern }));else if (match != null) {
                var form = this.state.forms[formName];
                if (form && form.values[match] != value) errors.push(this.createValidationError('match', { value: value, match: match }));
            } else {
                if (type === 'email' && !EMAIL_REGEX.test(value)) errors.push(this.createValidationError('email', { value: value }));
                if (type === 'number') {
                    if (integer && !INTEGER_REGEX.test(value)) errors.push(this.createValidationError('integer', { value: value }));
                    if (!integer && !FLOAT_REGEX.test(value)) errors.push(this.createValidationError('float', { value: value }));
                    if (min != null && +value < min) errors.push(this.createValidationError('min', { value: value, min: min }));
                    if (max != null && +value > max) errors.push(this.createValidationError('max', { value: value, max: max }));
                } else if (type === 'file' && files && files.length) {
                    files.forEach(function (file) {
                        if (extensions && extensions.length) {
                            var matchedExtensions = extensions.filter(function (extension) {
                                var regex = new RegExp('\\.' + extension.replace(/\./, '') + '$', 'i');
                                return regex.test(file.name);
                            });
                            if (!matchedExtensions.length) errors.push(_this6.createValidationError('extension', { value: value, file: file, extensions: extensions }));
                        } else if (accept) {
                            var matchedAccepts = accept.replace(/ /g, '').split(',').filter(function (mimetype) {
                                return new RegExp('^' + mimetype.replace(/\*/g, '.*') + '$').test(file.type);
                            });
                            if (!matchedAccepts.length) errors.push(_this6.createValidationError('accept', { value: value, file: file, accept: accept }));
                        }
                        if (maxSize != null && file.size > +maxSize) errors.push(_this6.createValidationError('maxSize', { value: value, file: file, maxSize: maxSize }));
                    });
                } else {
                    if (minLength != null && value && value.length < minLength) errors.push(this.createValidationError('minLength', { value: value, minLength: minLength }));
                    if (maxLength != null && value && value.length > maxLength) errors.push(this.createValidationError('maxLength', { value: value, maxLength: maxLength }));
                }
                if (validate.length && Object.keys(customValidators).length) {
                    validate.forEach(function (validatorSpec) {
                        var validatorName = validatorSpec;
                        var params = _extends({ value: value }, validatorSpec.params);
                        if ((typeof validatorSpec === 'undefined' ? 'undefined' : _typeof(validatorSpec)) === 'object') {
                            validatorName = validatorSpec.name;
                        }
                        var validator = customValidators[validatorName];
                        if (validator && !validator(value, params, files)) errors.push(_this6.createValidationError(validatorName, params));
                    });
                }
            }

            fieldCtrl.value = value;
            fieldCtrl.errors = errors;
            fieldCtrl.unchanged = value === initialValue;
            fieldCtrl.changed = value !== initialValue;
            fieldCtrl.valid = errors.length === 0;
            fieldCtrl.invalid = errors.length > 0;
        }
    }, {
        key: 'onFormSubmited',
        value: function onFormSubmited(formName, formRef) {
            this.setState(function (state) {
                var form = state.forms[formName];
                if (form) {
                    FormEventDispatcher.forwardSubmitFormEvent(formName, form.values, form, formRef);
                }
                return state;
            });
        }
    }, {
        key: 'onFormReseted',
        value: function onFormReseted(formName) {
            var _this7 = this;

            this.setState(function (state) {
                var forms = _extends({}, state.forms);
                var form = forms[formName];
                Object.keys(form.fields).forEach(function (fieldName) {
                    var field = form.fields[fieldName];
                    _this7.updateFieldCtrl(formName, field, field.initialValue);
                    field.touched = false;
                    field.untouched = true;
                    field.dirty = false;
                    field.pristine = true;
                    FormEventDispatcher.forwardFieldChangedEvent(formName, fieldName, field);
                    form.values[fieldName] = field.initialValue;
                });
                _this7.updateFormCtrl(formName, form);
                return state;
            });
        }
    }, {
        key: 'unsubscribe',
        value: function unsubscribe() {
            document.removeEventListener(REACT_FORMCTRL.EVENTS.REGISTER_FORM, this.onEvent);
            document.removeEventListener(REACT_FORMCTRL.EVENTS.REGISTER_FIELD, this.onEvent);
            document.removeEventListener(REACT_FORMCTRL.EVENTS.UNREGISTER_FORM, this.onEvent);
            document.removeEventListener(REACT_FORMCTRL.EVENTS.UNREGISTER_FIELD, this.onEvent);
            document.removeEventListener(REACT_FORMCTRL.EVENTS.FIELD_CHANGED, this.onEvent);
            document.removeEventListener(REACT_FORMCTRL.EVENTS.FIELD_BLURRED, this.onEvent);
            document.removeEventListener(REACT_FORMCTRL.EVENTS.FIELD_PROPS_CHANGED, this.onEvent);
            document.removeEventListener(REACT_FORMCTRL.EVENTS.FORM_SUBMITED, this.onEvent);
            document.removeEventListener(REACT_FORMCTRL.EVENTS.FORM_RESETED, this.onEvent);
        }
    }, {
        key: 'render',
        value: function render() {
            var children = this.props.children;

            return children;
        }
    }]);

    return FormProvider;
}(_react2.default.Component);

FormProvider.propTypes = {
    customValidators: _propTypes2.default.arrayOf(_propTypes2.default.shape({
        name: _propTypes2.default.string.isRequired,
        validate: _propTypes2.default.func.isRequired
    }))
};